<?php
namespace Twoir;
/**
 * 省市区 简单筛选adcode版
 * 高级可订制 pinyin、区号等
 * 2021.11.02 cighsen02 365182575
 * 更新 2022.05.28
 **/

use Twoir\System\Model\Enewsf;
use Twoir\System\Model\EnewsClass;
use Config;
use Exception;

class AllCity
{
	const VERSION 	= '1.0.3';
	//当前省份
	protected $province;
	//当前城市
	protected $city;
	//当前地区
	protected $district;
	//当前城市
	protected $adcode;//城市代码
	//当前classid
	protected $classid;
	//当前id
	protected $id;
	//所有省和市 键和值 
	protected $adcodetocname;
	//api时存放变量
	protected $adcodecname;
	//当时顶级栏目 dna、news等
	protected $column;
	
	protected $current;
	//所有栏目集合
	protected $columns;
	//filter 所有筛选集合
	protected $params; 
	//当前调用参数(url选择时值)
	protected $urls;
	//执行类型 api、web
	protected $method;
	//重写规则
	protected $rewrite;
	
	protected $allowfield;
	//开启模块
	protected $model; //开通模块
	
	const LEVEL = ['province' => 1, 'city' => 2, 'district' => 3];
	
	//特殊城市
	const SPCL_CITY = [4190, 4290, 4419, 4420, 4604, 4690, 6202, 6590];
	
	/**
	 * @param $adcodecname array, 
	 * @param $option array,
	 * @param $api string api|web 两种方式
	 */
	private function __construct($adcodecname = [], $option = [], $api = 'api')
	{
		$this->params = array_filter($option, function($value, $key) {
		    
		    if (!empty($value) && in_array($key, ['adcode', 'rewrite', 'allowfield', 'model', 'column', 'classid', 'id'])) $this->{$key} = is_numeric($value) ? (int)$value : $value;
			
		    return !('adcode' === $key || empty($value) || is_array($value)); 
		    
		}, ARRAY_FILTER_USE_BOTH);
		
		//分割myarea值
		if ($this->params['myarea'] && !in_array(substr($this->params['myarea']), self::SPCL_CITY)) {
			
			$this->params['myarea'] = substr($this->params['myarea'], 0, 4);
			$this->params['area']	= substr($this->params['myarea'], -2);
		}
		
		//请求方式
		$this->method = $api;

        //api方式存放数组，方便调取数据，提高效率
		if('api' === $api) $this->adcodecname = $adcodecname;
		
		//初始化
		$this->initialize($adcodecname);
	}
	
	/**
	 * 初始化
	 * @param $adcodecname array
	 */
	protected function initialize($adcodecname = [])
	{
		//初始化
		isset($this->current) and $this->current = null;
		
		//是否常住内存
		$adcodecname = isset($adcodecname) && !empty($adcodecname) ? $adcodecname : $this->adcodecname;
		
		//执行并返回当前城市集合
		$this->adcodetocname = $this->adcodeToCname($adcodecname, 2);
		
		//初始默认值
	    $this->urls = $this->params;
		
		//所有栏目
		$this->columns = $this->column();
	
		//只有栏目才执行筛选, 其它不执行
		if (isset($this->classid) && !isset($this->id)) {
			
			$this->tabs = $this->tabFilter();
		}
	}
	
	/**
	 * 分站核心，展开集合和当前位置
	 * @return 所有城市 参数为2 所有为0
	 */
	protected function adcodeToCname($adcodecname, $level = 2)
	{
		$result = [];
		array_walk_recursive($adcodecname, function ($cname, $adcode) use (&$result, $level) {

			//省级
			if(1 === $this->level($adcode)) {
				
				//所有省
				if (0 === $level || 1 === $level) $result[$adcode] = $cname;
				//香港、澳门复制父对象到城市上
				if (in_array(substr($adcode, 0, 2), [81, 82])) $result[$adcode] = $cname;
				
				//当前省集合
				$this->province[$adcode]= $cname;
				
				//当前省
				if(substr($adcode, 0, 2) === substr($this->adcode, 0, 2)) {
					
					$this->current['province'] = ['adcode' => $adcode, 'cname' => $cname];
					//香港、澳门复制父对象到城市上
		            if (in_array(substr($this->adcode, 0, 2), [81, 82])) {
		                
		                $this->current['city'] = ['adcode' => $adcode, 'cname' => $cname];
		                $result[$adcode] = $cname;
		                
		                isset($this->current['city']['start']) or $this->current['city']['start'] = $adcode;
		            }
					
					//省区间的起点
				    isset($this->current['province']['start']) or $this->current['province']['start'] = $adcode;
				}
			} 
			//市级
			elseif (2 === $this->level($adcode)) {
				
				//所有市
				if (0 === $level || 2 === $level) $result[$adcode] = $cname;
			    
			    //当前市集合
			    if(substr($adcode, 0, 2) === substr($this->adcode, 0, 2)) $this->city[$adcode] = $cname;
				
				//当前市(含特殊城市)
				if($adcode === $this->adcode || (!in_array(substr($adcode, 0, 4), self::SPCL_CITY) && substr($adcode, 0, 4) === substr($this->adcode, 0, 4))) {
					
					$this->current['city'] = ['adcode' => $adcode, 'cname' => $cname];
					//市区间的起点
				    isset($this->current['city']['start']) or $this->current['city']['start'] = $adcode;
				}
			} 
			//县区
			else {
                
                //所有区县
                if (0 === $level || 3 === $level) $result[$adcode] = $cname;
				
				//省区间的终点
				if (substr($adcode, 0, 2) === substr($this->adcode, 0, 2)) {
				    
				    $this->current['province']['end'] = $adcode;
				    //除香港、澳门特殊处理外 //市区间的终点
					if (in_array(substr($adcode, 0, 2), [81, 82])) $this->current['city']['end'] = $adcode;
				}
                
                //当前区县集合
				if (substr($adcode, 0, 4) === substr($this->adcode, 0, 4)) {
				                    
					$this->district[$adcode] = $cname;
					//除香港、澳门特殊处理外 //市区间的终点
					if (!in_array(substr($adcode, 0, 2), [81, 82])) $this->current['city']['end'] = $adcode;
				}
			    
				//当前县区
				if($adcode === $this->adcode) $this->current['district'] = ['adcode' => $adcode, 'cname' => $cname];
			}
		});
		
		//省
        if ($this->adcode > 100000 && $this->level($this->adcode) === 1 && !array_key_exists($this->adcode, $this->province)) throw new Exception('adocde参数错误', 9401);
		//城市
		if ($this->adcode > 100000 && $this->level($this->adcode) === 2 && !array_key_exists($this->adcode, $this->city)) throw new Exception('adocde参数错误', 9401);
        //区县
		if ($this->adcode > 100000 && $this->level($this->adcode) === 3 && !array_key_exists($this->adcode, $this->district)) throw new Exception('adocde参数错误', 9401);
	
		return $result;
	}
	
	/*
	* 返回城市级别 2为城市
	*/
	protected function level($code = 0)
	{
		if (substr($code, 0, 2) > 10 && substr($code, -4) < 1) {
			
			$level = 1;
		} 
		elseif ((substr($code, 2, 2) > 0 && substr($code, -2) < 1) || in_array(substr($code, 0, 4), self::SPCL_CITY)) {
			
			$level = 2;
		}
		elseif (substr($code, -2) > 0) {
			
			$level = 3;
		} 
		else {
			
			$level = 0;
		}
		
		return $level;
	}
	
	/**
	 * 返回当前伪静规则
	*/
	protected function rewrite($add)
	{
		$modle = isset($add['column']) ? $add['column'] : $this->column;

		return isset($modle, $this->rewrite[$modle]) ? $this->rewrite[$modle] : [];
	}
	
	/**
	 * @param 通过栏目名转classid
	 * @return classid 
	 */
	protected function column2classid($column = null)
	{
		//支持自动获取 
		$column = isset($column) ? $column : $this->params['column'];
		
		if (isset($column) && isset($this->columns[0]) && in_array($column, $this->model)) {
			
			foreach($this->columns[0] as $classid => $item) {
				
				if ($item['model'] === $column) return $classid;
			}
		}

		return 0;
	}
	
	/**
	 * 通过classid找栏目
	*/
	protected function classid2column($classid = null)
	{
		//大栏目名称
		if (isset($classid, $this->columns)) {
		    
			array_walk($this->columns, function($carray, $id) use (&$classid){
				
				if (array_key_exists($classid, $carray)) $classid = $carray[$classid]['model'];
			});
		}

		return isset($classid) ? $classid : 0;
	}
	
	/**
	 * 当前是否活动项(高亮)
	 */
	protected function preActive($add = [])
	{
		if (!empty($add)) {
			
			$add = array_filter($add, function($item){ return $item['active'];});
		}
		
		return $add;
	}
	
	/**
	 * 字符串分割成数组
	 */
	protected function stringExplode($data)
	{
		if (!isset($data) || empty($data)) return [];
		
		$vr 		= empty($data['fvalue']) ? [] : explode('|', $data['fvalue']);
		$count 		= count($vr);
		$def 		= ':default';
		$newdata 	= [];
		
		//价格区间
		if ('price' === $data['f']) {
	
			//检测是否存在
			if (isset($this->params[$data['f']]) && !array_key_exists($this->params[$data['f']], self::PRICE)) throw new Exception('price参数错误', 9401);
			
			foreach(self::PRICE as $id => $text) {

				$newdata[$id] = [
					'name'	=> $text,
					'url'	=> $this->resolveUrl([$data['f'] => $id], 'list'),
					'active'=> (string)$id === $this->params[$data['f']]
				];
			}
		}
		
		//地区
		elseif ('myarea' === $data['f']) {
		
			$newdata = $this->url($this->district, 3, true);
		}
		
		else {
	
			for($i=0; $i < $count; $i++){
				
				$isdef = false;
				if(false !== strstr($vr[$i], $def)){
					$dr     = explode($def, $vr[$i]);
					$vr[$i] = $dr[0];
					$isdef  = true;
				}
	
				$selectvalr     = explode('==', $vr[$i]);
				$selectvalr[0]  = is_numeric($selectvalr[0]) ? (int)$selectvalr[0] : $selectvalr[0];
				
				// 过滤0/00不显示前台
				if (0 < $selectvalr[0]) {

					$newdata[$selectvalr[0]] = [
						'name' 	=> $selectvalr[1] ? $selectvalr[1] : $selectvalr[0],
						'url'	=> $this->resolveUrl([$data['f'] => $selectvalr[0]], 'list'),
						'active'=> (string)$selectvalr[0] === $this->params[$data['f']],
					];
				}
			}
			
			//检测是否存在
			if (isset($this->params[$data['f']]) && !array_key_exists($this->params[$data['f']], $newdata)) throw new Exception($data['f'] . '参数错误', 9401);
		}

		return $newdata;
	}
	
	/**
	 * 转成链接
	 */
	protected function url($urls, $level = 1, $showActive = false)
	{
		//字符串强转数组
		if (!is_array($urls)) $urls = [];
		
		$newUrls = [];
		foreach($urls as $adcode => $cname) {
			
			if (isset($this->urls['area']) && $level < 3) unset($this->urls['area']);
			
			//特殊城市处理
			if(2 === $level && substr($adcode, -2) > 0) {
				
				$id = $adcode;
				$newUrls[$id] = [
					'fullname'	=> $cname,
					'cname' 	=> $this->shortName($cname, $level),
					'url'		=> $this->resolveUrl(['myarea' => $id], 'list')
				];
				//高亮
				if ($showActive) $newUrls[$id]['active'] = $adcode === $this->adcode;
			}
			elseif(3 === $level){
				
				$id = (int)substr($adcode, -2);
				$newUrls[$id] = [
					'fullname'	=> $cname,
					'cname' 	=> $this->shortName($cname, $level),
					'url'		=> $this->resolveUrl(['myarea' => substr($adcode, 0, 4), 'area' => $id], 'list'), 
				];
				//高亮
				if ($showActive) $newUrls[$id]['active'] = $adcode === $this->adcode;
			}
			else{
				
				$id = 1 === $level ? str_pad(substr($adcode, 0, 2), 4, 0) : substr($adcode, 0, 4);
				$newUrls[$id] = [
					'fullname'	=> $cname,
					'cname' 	=> $this->shortName($cname, $level),
					'url'		=> $this->resolveUrl(['myarea' => $id], 'list'),
				];
				//高亮
				if ($showActive) $newUrls[$id]['active'] = $id === (1 === $level ? str_pad(substr($this->adcode, 0, 2), 4, 0) : substr($this->adcode, 0, 4));
			}
		}
		
		return $newUrls;
	}
	
	/**
	 * 通过配制文件解析url，支持筛选参数可随意调换 stop 提前退出
	 * @return url
	 */
	protected function resolveUrl($urls = [], $pageType = 'list', $stop = '', $remove = [])
	{
		global $class_r;
		
		$urls = $urls ? array_replace($this->urls ? $this->urls : [], $urls) : $this->urls;
	    
	    //移除指定值
		if (!empty($urls) && !empty($remove)) {
		    
		    $urls = array_filter($urls, function($key) use ($remove) {

		        return !in_array($key, $remove);
		        
		    }, ARRAY_FILTER_USE_KEY);
		}
		
		$rewrite = $this->rewrite($urls);
	
		//详情页
		if ('page' === $pageType && isset($rewrite[$pageType], $rewrite['pagefield'])) {
			
			foreach($rewrite['pagefield'] as $key) {
				
				$rewrite[$pageType] = str_replace('[!--' . $key . '--]', $urls[$key], $rewrite[$pageType]);
			}
		} 
		//列表页
		elseif ('list' === $pageType && isset($rewrite[$pageType], $rewrite['listfield'])) {	
			
			//翻页伪静后缀
			$suf = '';
			if (false !== strstr($rewrite[$pageType], '[!--page--]')) {
				
				$suf = substr(strrchr($rewrite[$pageType], '/'), 1);
				$rewrite[$pageType] = strstr($rewrite[$pageType], $suf, true);
			}
			
			//退出标志符
			$flag   = false;
			$i      = 0;
			$af     = true;
			foreach($rewrite['listfield'] as $key) {
				
				//去除没有选择中项
				if (!isset($urls[$key]) || $flag) {
                
					if (in_array($key, $rewrite['listfield']) && $af && 'myarea' !== $key && 'column' !== $key) {
						
						++$i;
						$af = false;
					}

					$pos = strpos($rewrite[$pageType], '[!--' . $key . '--]') - $i;
					$len = strlen('[!--' . $key . '--]') + $i + ('myarea' === $key ? 1 : 0);
					$rewrite[$pageType] = substr_replace($rewrite[$pageType], '', $pos, $len);
				} 
				else {
					
					$rewrite[$pageType] = str_replace('[!--' . $key . '--]', $urls[$key], $rewrite[$pageType]);
				}
				
				//退出
				if ($key === $stop) $flag = true;
			}
		
			//只有区域去，去除最后的/
			$rewrite[$pageType] = rtrim($rewrite[$pageType], '/') . '/';
            
            //有翻页码加上
			if ('page' === $stop && '' !== $suf) {
				
				$rewrite[$pageType] = $rewrite[$pageType] . str_replace('[!--page--]', $urls['page'], $suf);
			}
		}
		//首页
		elseif (isset($urls['myarea'])) {
			
			$rewrite[$pageType] = str_replace('[!--myarea--]', $urls['myarea'], '[!--myarea--]/');
		}
	
		//重置默认
		$this->urls = $this->params;
		
		return '/' . ltrim($rewrite[$pageType], '/');
	}
	
	
	//1省 2市 3区、县
	protected function shortName($text = '', $level = 2)
	{
		//省
		if(1 === $level){
			
			$text = str_replace(['自治区', '壮族', '回族', '维吾尔', '特别行政区', '省', '市'], '', $text);
		}
		//市、区
		elseif(2 === $level){
			//基本替换
			$text = str_replace(
			[
				'市辖区', '自治州', '朝鲜族', '土家族', '苗族', '瑶族', '藏族', '羌族', '彝族', '黎族', '哈萨克', '布依族', '侗族', 
				'哈尼族', '傣族', '壮族', '白族', '景颇族', '傈僳族', '回族', '蒙古族', '特别行政区', '自治县', '林区', '地区', '市'
			], 
			'', 
			$text);
			
			//特殊替换
			$text = str_replace(['克孜勒苏柯尔克孜',	'巴音郭楞蒙古', '博尔塔拉蒙古', '锡林郭勒盟', '阿拉善盟'], ['克州', '巴州', '博州', '锡林郭勒', '阿拉善'], $text);
		}
		
		//区县，不简称
		elseif (3 === $level) {
			
			//两个字不替换
			if(mb_strlen($text) < 3) return $text;

			//特列替换
			if(in_array($text, ['新市区', '西市区', '沙县区']))	return str_replace('区', '', $text);

			//基本替换
			$text = str_replace(['自治旗', '自治县', '蒙古', '区', '县', '市'], '', $text);
		}
		
		return $text;
	}
	
	/*
	 * 分页码
	 * 2021.04.07 新增 cighsen02 365182575
	*/
	public function navPage($pagesize = 10)
	{
		$res        = [];
		$pagesize 	= isset($GLOBALS['line']) ? $GLOBALS['line'] : $pagesize; //设置系统分页
		$page 		= isset($this->params['page']) ? (int)$this->params['page'] : 1;
		$num		= isset($GLOBALS['num']) && $GLOBALS['num'] > 0 ? $GLOBALS['num'] : 0;
		$totalpage 	= $num > $pagesize ? ceil($num / $pagesize) : 1;
	
		$showpage = 8;
		$leftpage = floor($showpage / 2);
		
		//左边显示数为当前页减左边该显示的数 例如总显示7页 当前页是5 左边最小为5-3 右边为5+3
		$rightpage 	= $showpage - $leftpage;
		$left 		= max($page - $leftpage, 1); //左边最小不能小于1
		$right 		= $left + $showpage - 1; //左边加显示页数减1就是右边显示数
		$right 		= (int)min($right, $totalpage); //右边最大不能大于总页数
		$left 		= (int)max($right - $showpage + 1, 1); //确定右边再计算左边，必须二次计算
		
		//伪静外询查参数
		$qs = parse_url($_SERVER['REQUEST_URI'], PHP_URL_QUERY);
		
		//上一页
		$res[0]['text']     = '上一页';
		$res[0]['url'] 	    = $page > 1 ? $this->resolveUrl(['page' => 1 === $page - 1 ? '' : $page - 1], 'list', 1 === $page - 1 ? '' : 'page') . ($qs ? '?' . $qs : '') : '';
		$res[0]['active']   = false;
		for ($i = $left; $i <= $right; $i++) {
			$res[$i]['text']    = $i;
			$res[$i]['url']     = $page !== $i ? $this->resolveUrl(['page' => 1 === $i ? '' : $i], 'list', 1 === $i ? '' : 'page') . ($qs ? '?' . $qs : '') : '';
			$res[$i]['active']  = $page === $i;
		}
		$res[$i]['text']    = '下一页';
		$res[$i]['url']     = $totalpage > $page ? $this->resolveUrl(['page' => $totalpage > $page + 1 ? $page + 1 : $totalpage], 'list', 'page') . ($qs ? '?' . $qs : '') : '';
		$res[$i]['active']  = false;
	
		return $totalpage > 1 ? $res : [];
	}
	
	/**
	 * 访问数据区
	 * 
	*/
	protected function classCache()
	{
		//实时获取，有需要缓存可以订制，最多取12000条，如果栏目超过这个数，自己改动一下
		$class_r = EnewsClass::getList(['showclass' => 0], ['classid', 'bclassid', 'classname', 'classpath', 'showclass'], 0, 12000);
		
		return array_reduce($class_r, function($carry, $item) {
			
			$model = false !== strstr($item['classpath'], '/') ? strstr($item['classpath'], '/', true) : $item['classpath'];
			
			//二级栏目
			if (false !== strstr($item['classpath'], '/')) {
				
				$url 	= in_array($model, $this->model) ? $this->resolveUrl(['column' => strstr($item['classpath'], '/', true), 'classid' => $item['classid']], 'list', 'classid') : '/' . (isset($this->params['myarea']) ? $this->params['myarea'] . '/' : '') . $item['classpath'] . '/';
				$id 	= $item['classid'];
		
				if ($this->classid === $id) {
					
					$this->current['class'] = ['name' => $item['classname'], 'url' => $url];
				}
			}
			//一级栏目
			else {
				
				$url 	= in_array($model, $this->model) ? $this->resolveUrl(['column' => $item['classpath']], 'list', 'column') : '/' . (isset($this->params['myarea']) ? $this->params['myarea'] . '/' : '') . $item['classpath'] . '/';
				$id 	= $item['classpath'];
		
				if ($this->params['column'] === $id) {
					
					$this->current['column'] 	= ['name' => $item['classname'], 'url' => $url];
				}
			}
			
			$carry[$item['bclassid']][$item['classid']] = [
				'name'	=> $item['classname'],
				'url'	=> in_array($model, $this->model) ? $url : '/' . $item['classpath'] . '/',
				'id'	=> $id,
				'model'	=> $model,
				'showclass' => $item['showclass'],
				'active'=> $this->classid === $id || $this->params['column'] === $id
			];
	
			return $carry;
			
		}, []);
	}
	
	/**
	 * 筛选操作
	 */
	protected function tabFilter($column = null)
	{
		isset($column) or $column = $this->column;
		
		if (!isset($column) || empty($field = $this->listandf($column))) {
			
			return [];
		}
		
		//访问数据库
		$r = Enewsf::getList(['tbname' => $column, 'f in' => $field], ['tid', 'f', 'fvalue', 'fform', 'fname']);

		if (empty($r)) return [];
	
		if(!isset($this->district) || 'combo' === $column) {

			$r = array_filter($r, function($value) { return 'myarea' !== $value['f'];});
		}
		
		$result = [];
		array_walk($r, function ($value, $key) use (&$result) {
			
			$main   = $this->stringExplode($value);
			$active = $this->preActive($main);
			if(!empty($active) && 'myarea' !== $value['f']) {
				
				$this->current[$value['f']] = [
					'name'	=> $active[$this->params[$value['f']]]['name'],
					'url'	=> $this->resolveUrl([], 'list', '', [$value['f']]) 
				];
			}
			
			$result[$value['f']] = [
				'text'	=> $value['fname'],
				'main' 	=> $main,
				'other'	=> [
					'name'	=> 'myarea' == $value['f'] ? (isset($active[$this->params['area']]['fullname']) ? $active[$this->params['area']]['fullname'] : '不限') : (isset($active[$this->params[$value['f']]]['name']) ? $active[$this->params[$value['f']]]['name'] : '不限'),
					'url'	=> 'myarea' == $value['f'] ? $this->resolveUrl([], 'list', '', ['area']) : $this->resolveUrl([], 'list', '', [$value['f']]),
					'active'=> empty($active)
				]
			];
		});
		//释放
		unset($r);
		
		return $result;
	}
	
	/**
	 * 取结合项字段名
	 * 
	 */
	protected function listandf($column = null)
	{
		global $dbtbpre, $empire, $emod_r, $etable_r;
		
		if (!isset($column) && !isset($this->column)) return [];
		
		isset($column) or $column = $this->column;
		
		//取缓存
		if (!isset($etable_r[$column]) || (0 === (int)$mid = $etable_r[$column]['mid']) || !isset($emod_r[$mid]['listandf']) || empty($listandf = $emod_r[$mid]['listandf'])) {
			
			//查库待加
		}
		
		return empty($listandf) ? [] : explode(',', trim($listandf, ','));
	}
	
	//访问数据区结束
   
	/**
	 * 
	 */
	protected function column()
	{
		global $class_r;
		
		if(empty($classes = $this->classCache()) || empty($classes[0])) throw new Exception('获取栏目出错了', 9401);

		//栏目空是，获取栏目 column  
		if (isset($this->classid) && !isset($this->column)) {

			array_walk($classes, function($carray, $classid){
	 			
				if (array_key_exists($this->classid, $carray)) {
                  
					$this->column = $this->params['column'] = $this->urls['column'] = $carray[$this->classid]['model'];
				}
			});

			if (!isset($this->params['column'])) throw new Exception('classid2参数错误', 9401);
		}
		
		//没有值进行赋值
		if ((!isset($this->current['column']) || !isset($this->classid)) && $this->column) {
			
			array_walk($classes[0], function($carray, $classid) {
				
				if ($carray['id'] === $this->column) {
                    
                    //classid值
                    if (!isset($this->classid)) {
                        
                        $this->classid = $classid;
                    }
				    
				    //栏目值
				    if (!isset($this->current['column'])) {
				        
				        $this->current['column'] = [
        			    	'name'	=> $carray['name'],
        			    	'url'	=> $carray['url']
        			    ];
				    }
				}
			});
	
			if (!isset($this->classid))	throw new Exception('classid出错了', 9401);
		}
		
		// 检测classid是否存在
		if (isset($this->classid) && !isset($class_r[$this->classid])) throw new Exception('classid55参数错误', 9401);

		return $classes;
	}
	
	/**
	 * 参数设置
	 */
	public function setOption($option = [])
	{
		if (!isset($option) || 'api' !== $this->method) {
			
			throw new Exception('web工作模式不允许设置参数', 9401);
		}
		
		//过滤参数
		$option = array_filter($option, function($value, $key) { 
			
			if (empty($value) || is_numeric($key) || !in_array($key, ['myarea', 'classid', 'id'])) {
				
				return false;
			}
			
			if ('myarea' === $key && $value > 100000 && $this->adcode !== (int)$value) {
				
				$this->adcode = (int)$value;
				$this->params['myarea'] = in_array(substr($value, 0, 4), self::SPCL_CITY) ? $value : substr($value, 0, 4);
			}
			elseif ($value > 0 && $this->{$key} !== (int)$value) {
				
				$this->{$key} = (int)$value;
				if ('classid' === $key) $this->params['column'] = $this->classid2column($value);
			}
			
			return true;
			
		}, ARRAY_FILTER_USE_BOTH);
		
		unset($option);
	
		//重载
		$this->initialize();
	}
	

	//以下获取区 开始
	
	/**
	 * @retrun $adocde
	 */
	public function getAdcode()
	{
		return null === $this->adcode || 100000 === $this->adcode ? 0 : $this->adcode;
	}
	
	/**
	 * @retrun $classid
	 */
	public function getClassid()
	{
		return isset($this->classid) ? $this->classid : 0;
	}
	
	/**
	 * @retrun $id
	 */
	public function getId()
	{
		return isset($this->id) ? $this->id : 0;
	}
	
	/**
	 * 返回类型 指定链接[栏目地址或筛选地址]
	 * urlInfo
	 */
	public function getUrlArea($data = [], $level = 0)
	{
		if(!isset($data['myarea']) || 1 > $data['myarea'] || 2 > $this->level($data['myarea'])) return [];
	
		if(!isset($this->adcodecname)) $this->adcodecname = Config::twoir('adcodecname');
	
		$result = [];
		array_walk_recursive($this->adcodecname, function($v, $k) use (&$result, $data) {

        	//省级
        	if (1 === $this->level($k)) {
        	   
				if ($k === (int)str_pad(substr($data['myarea'], 0, 2), 6, 0)) {
				   
				    $id = (int)substr($k, 0, 4);
				    $result['province'] = [
				        'cname'     => $this->shortName($v, 1),
				        'fullname'  => $v,
				        'url'       => $this->resolveUrl(['myarea' => $id, 'column' => $this->classid2column($data['classid'])], 'list', 'column')
				    ];
				}
        	}
        	elseif (2 === $this->level($k)) {
        	    
        	    $adcode = str_pad(substr($data['myarea'], 0, 4), 6, 0);
        	    $adcode = 2 === $this->level($adcode) ? $adcode : $data['myarea'];
        	    if ($k === (int)$adcode) {
        	        
        	        $id = (int)substr($k, 0, 4);
            		$result['city'] = [
				        'cname'     => $this->shortName($v, 2),
				        'fullname'  => $v,
				        'url'       => $this->resolveUrl(['myarea' => $id, 'column' => $this->classid2column($data['classid'])], 'list', 'column')
				    ];
        	    }
        	} 
        	else {
        	    
        	    if ($k === (int)$data['myarea']) {
        	        
        	        $id = (int)substr($k, -2);
            		$result['district'] = [
				        'cname'     => $this->shortName($v, 3),
				        'fullname'  => $v,
				        'url'       => $this->resolveUrl(['myarea' => substr($k, 0, 4), 'column' => $this->classid2column($data['classid']), 'area' => $id], 'list', ['area'])
				    ];
        	    }
        	}
        });
		
		//名称
		$level = array_search($level, self::LEVEL);
			
		return $level ? $result[$level] : $result;
	}
	
	/**
	 * 获取城市路径
	 */
	public function getCityPath()
	{
	    if(!isset($this->adcode) || 100000 === $this->adcode || 2 > $this->level($this->adcode)) return '';
		
		return in_array(substr($this->adcode, 0, 4), self::SPCL_CITY) ? $this->adcode : substr($this->adcode, 0, 4);
	}
	
	/**
	 * 返回栏目链接
	 * 
	 */
	public function getUrl($option = [], $stop = '', $remove = [])
	{
		//去除无关参数
		$option = array_filter($option, function($value, $key) {
            
			return !empty($value) && !is_string($key) && in_array($key, $this->allowfield) && ('myarea' === $key && 100000 < $value);
			
		}, ARRAY_FILTER_USE_BOTH);

		//转换类型
		array_walk($option, function(&$value, $key) {
			
			if ('myarea' === $key) {
				
				$value = in_array(substr($value, 0, 4), self::SPCL_CITY) ? $value : substr($value, 0, 4);
			}
			else {
				
				$value = is_numeric($value) ? (int)$value : $value;
			}
		});
		
		//大栏目名称
		if (!isset($option['column']) && isset($option['classid']) && $option['classid'] > 0) {
		    
			$option['column'] = $this->classid2column($option['classid']);
		}

		return $this->resolveUrl($option, 'list', $stop, $remove);
	}
	
	/**
	 * $rep 改为自定义前缀
	 * 返回详情页链接
	 */
	public function getPageUrl($option = [], $rep = null)
	{
		//去除无关参数
		$option = array_filter($option, function($value, $key) { 

			return !(empty($value) || is_numeric($key) || !in_array($key, ['myarea', 'column', 'classid', 'id']) || ('myarea' === $key && 100000 >= $value));
			
		}, ARRAY_FILTER_USE_BOTH);
	    
		//类型转
		array_walk($option, function(&$value, $key) use ($rep) {
			
			if ('myarea' === $key) {
				
				$value = in_array(substr($value, 0, 4), self::SPCL_CITY) ? $value : substr($value, 0, 4);
			} 
			//支持改id
			elseif (null !== $rep && 'id' === $key) {
				
				$value = $rep . $value;
			}
			else {
				//转成数字
				$value = is_numeric($value) ? (int)$value : $value;
			}
		});
	
		//大栏目名称
		if (!isset($option['column']) && isset($option['classid']) && $option['classid'] > 0) {
		    
			$option['column'] = $this->classid2column($option['classid']);
		}

		return $this->resolveUrl($option, 'page');
	}
	
	
	public function getTabFilter($column, $field = null)
	{
		if (!isset($column) || empty($column)) {
			
			return [];
		}
		
		$d = [];

		if (!isset($this->tabs) && empty($d = $this->tabs)) {
			
			$d = $this->tabFilter($column);
		}
		
		//可考虑增加入内存里
		if(isset($field) && !empty($field) && in_array($field, $this->listandf($column))) {
			
			$d = $this->tabFilter($column);
		}
		
		array_walk($d[$field]['main'], function(&$value, $key) use ($column, $field) {
			
			$field = 'myarea' === $field ? 'area' : $field;
			
			$value = [
				'text' 	=> 'area' === $field ? $value['fullname'] : $value['name'],
				'url'	=> $this->resolveUrl(['column' => $column, $field => $key], 'list')
			];
		});
		
		return empty($field) ? [] : $d[$field]['main'];
	}
	
	/**
	 * tabs 筛选组合输出 
	**/
	public function getTabs()
	{
		$classid= $this->column2classid();
		$data   = [];
		
		//栏目执行更新
		if ($classid > 0 && isset($this->columns[$classid])) {
			
			$main = $this->columns[$classid];
		
			array_walk($main, function(&$carray, $item) {
		
				//更新url地址
				$carray['url'] 		= $this->resolveUrl(['classid' => $carray['id']], 'list');
				$carray['active'] 	= $this->classid === $carray['id'];
				unset($carray['model']);
			});
			
			$active = $this->preActive($main);
			
			$data['class'] = [
				'text'	=> 'dna' === $this->params['column'] ? '机构等级' : '检测项目',
				'main' 	=> $main,
				'other'	=> [
					'name'	=> isset($active[$this->classid]['name']) ? $active[$this->classid]['name'] : '不限',
					'url'	=> $this->resolveUrl([], 'list', '', ['classid']),
					'active'=> empty($active)
				]
			];
		}
		
		//合并
		if (isset($this->tabs)) {
		    
		    $data = array_merge($data, $this->tabs);
		}

		return $data;
	}
	
	//创建目录 $level 返回层级数
	public function getCurrent($level = 2)
	{
		//当前城市
		if (isset($this->current) && !empty($city = $this->current)) {
			
			array_walk($city, function(&$value, $key) {
				
				//地区
				if (array_key_exists($key, self::LEVEL)) {
					
				    $value = [
				        'fullname'  => $value['cname'],
				        'cname'     => $this->shortName($value['cname'], self::LEVEL[$key]),
				        'url'       => $this->level($value['adcode']) > 2 ? $this->resolveUrl(['myarea' => substr($value['adcode'], 0, 4), 'area' => (int)substr($value['adcode'], -2)], 'list') : $this->resolveUrl(['myarea' => substr($value['adcode'], 0, 4)], 'list'),
				        'sql'       => isset($this->current[$key]['end'], $this->current[$key]['start']) ? ' myarea BETWEEN ' . $this->current[$key]['start'] . ' AND ' . $this->current[$key]['end'] : (isset($this->current[$key]['start']) ? ' myarea = ' . $this->current[$key]['start'] : ' myarea = ' . $value['adcode'])
				    ];
				}
			});
		}

		//城市
		if($this->district && $level >= 3){
			$city['cities'] = [
				'province' 	=> $this->url($this->province, 1, true),
				'city' 		=> $this->url($this->city, 2, true),
				'district' 	=> $this->url($this->district, 3, true),
			];
		} elseif($this->city && $level >= 2) {
			$city['cities'] = [
				'province' 	=> $this->url($this->province, 1, true),
				'city' 		=> $this->url($this->city, 2, true),
			];
		} else {
			$city['cities'] = [
				'province' 	=> $this->url($this->province, 1, true),
			];
		}
		
		//当前adocde
		$city['adcode'] = $this->adcode;
		
		//栏目
		$city['columns'] = $this->columns;
		
		//去除隐藏项
		array_walk($city['columns'], function(&$value, $key) {
		    
		    $value = array_filter($value, function($item) {
		        
		        return 0 === (int)$item['showclass'];
		    });
		    
		});

		return $city;
	}
	
	//以下获取区 结束
	
	/**
	 * 预处理伪静
	*/
	public static function preRewrite($add = [])
	{
		$fields = [];
		array_walk($add['rewrite'], function(&$v, $k) use (&$fields) {
			
			if (preg_match_all("/--(\w+)--/", $v['list'], $field) && isset($field[1])) {
				
				array_pop($field[1]);
				$v['listfield'] = $field[1];
				$fields         = array_merge($fields, $field[1]);
			}
			
			if (preg_match_all("/--(\w+)--/", $v['page'], $field) && isset($field[1])) {
				
				$v['pagefield'] = $field[1];
				$fields         = array_merge($fields, $field[1]);
			}
		});
		
		$add['allowfield'] = array_unique($fields);
		
		return $add;
	}

	/**
	 * 参数通过web方式输入 接收两个参数
	 * initialize
	 */
	public static function webStart()
	{
		//加载配制文件
		$config = Config::twoir(['adcodecname', 'allcityconf']);
		
		if(empty($config['adcodecname']) || empty($config['allcityconf'])) {
			
			throw new Exception('配制文件不能为空', 9401);
		}
		
		//预处理伪静
		$config['allcityconf'] 	= self::preRewrite($config['allcityconf']);

		//参数过滤处理
		$option = array_filter($_GET, function($value, $key) use($config) {
			
			if (empty($value) || !in_array($key, $config['allcityconf']['allowfield'])) {
				
				return false;
			}
			
			//错误处理
			if (in_array($key, ['myarea']) && (4 > strlen($value) || !is_numeric($value))) {
				
				throw new Exception($key . '参数错误', 9401);
			} 
			elseif ('area' === $key && (2 < strlen($value) || !is_numeric($value))) {
				
				throw new Exception($key . '参数错误', 9401);
			} 
			elseif ('column' === $key && is_numeric($value)) {
				
				throw new Exception($key . '参数错误', 9401);
			}
			elseif ('page' === $key && !is_numeric($value)) {
				
				throw new Exception($key . '参数错误', 9401);
			}
			
			return true;
			
		}, ARRAY_FILTER_USE_BOTH);
		
		//详情页，没有栏目id
		if(isset($option['id']) && $option['id'] > 0 && !isset($option['classid']) && !isset($option['column'])) {
			
			throw new Exception('classid参数错误', 9401);
		}
	
		//adcode转换6位，如area有值加上
		if (isset($option['myarea']) && $option['myarea']) {
			
			if (4 === strlen($option['myarea']) && isset($option['area']) && $option['area'] > 0) {
				
				$option['adcode'] = $option['myarea'] . str_pad($option['area'], 2, 0, STR_PAD_LEFT);
			} else {
				
				$option['adcode'] = str_pad($option['myarea'], 6, 0);
			}
		}
		
		else {
			
			//如果存在area异常
			if (isset($option['area']) && $option['area'] > 0) throw new Exception('area参数错误', 9401);
			
			//默认值
			$option['adcode'] = 100000;
		}

		return new static($config['adcodecname'], array_merge($option, $config['allcityconf']), 'web');
	}
	
	/**
	 * 参数通过api方式输入
	 * adcode 
	 * initialize
	 */
	public static function apiStart($option = [])
	{
		//加载配制文件
		$config = Config::twoir(['adcodecname', 'allcityconf']);
		
		if(empty($config['adcodecname']) || empty($config['allcityconf'])) throw new Exception('配制文件不能为空', 9401);
		
		//预处理伪静
		$config['allcityconf'] 	= self::preRewrite($config['allcityconf']);
	
		//去除无关参数
		$option = array_filter($option, function($value, $key) use ($config) { 
			
			if (empty($value) || !in_array($key, $config['allcityconf']['allowfield'])) return false;
			
			//错误处理
			if ('myarea' === $key && (4 > strlen($value) || !is_numeric($value) || $value < 100000)) throw new Exception($key . '参数错误', 9401);
			
			return true;
			
		}, ARRAY_FILTER_USE_BOTH);
		
		//详情页，没有栏目id
		if(isset($option['id']) && $option['id'] > 0 && !isset($option['classid'])) throw new Exception('classid参数错误', 9401);
		
		//adcode转换6位，如area有值加上
		if (isset($option['myarea'])) {
			
			$option['adcode'] = $option['myarea'];
			$option['myarea'] = in_array(substr($option['myarea'], 0, 4), self::SPCL_CITY) ? $option['myarea'] : substr($option['myarea'], 0, 4);
		}
		//默认值
		else {
			
			$option['adcode'] = 100000;
		}

		return new static($config['adcodecname'], array_merge($option, $config['allcityconf']), 'api');
	}
}